package de.hawhamburg.reachability.ui.frame;

import java.awt.Cursor;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.hawhamburg.lifecycle.data.ConfigOrder;
import de.hawhamburg.lifecycle.data.JsonConfigOrder;
import de.hawhamburg.livingplace.messaging.activemq.wrapper.LPPublisher;
import de.hawhamburg.reachability.ProjectInfo;
import de.hawhamburg.reachability.conf.MainConfig;
import de.hawhamburg.reachability.ui.UI;
import de.hawhamburg.reachability.ui.enumeration.FieldType;
import de.hawhamburg.reachability.ui.frame.resource.DisplayParameter;
import de.hawhamburg.reachability.util.ConnectionUtil;
import de.hawhamburg.reachability.util.EnumerationUtil;
import de.hawhamburg.reachability.util.OutputUtil;
import de.hawhamburg.reachability.util.XMLUtil;

/**
 * UI: A user interface to send control messages via the ActiveMQ.<br><br>
 * 
 * This class opens a new user interface which presents a formula in which the developer may specify an ID and a 
 * command (related to the living place agent life cycle specifications: start, stop, sleep, info) and sends this
 * control message via the ActiveMQ topic.
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public class ControllingToolUI extends UI {
    
    /**
     * Some unique serial version id.
     */
    private static final long serialVersionUID  = 4758643996143943024L;
    
    /**
     * The {@link LPPublisher}, which sends control messages via the ActiveMQ globel control topic.
     */
    private LPPublisher globalControlTopic      = null;
    
    /**
     * The parameters, which can be edited by the user.
     */
    private Map<String, DisplayParameter> parameters;
    
    /**
     * Creates and builds a new {@link ControllingToolUI} and opens all relevant ActiveMQ connections.
     */
    public ControllingToolUI() {
        super("Controlling Tool");
        build();
        globalControlTopic = ConnectionUtil.getPublisher(MainConfig.get("LP_CONTROL_TOPIC"), this);
    }
    
    /**
     * Loads the local configuration of this user interface from its XML file. The rendering of this user interfaces is
     * largely defined by this configuration file.
     * 
     * @param configFile    The path to the configuration file. 
     */
    protected final void loadLocalConfig(final String configFile) {
        Document config = XMLUtil.loadDocument(configFile);
        
        String xParameters = "/configuration/rendering/parameters/parameter";
        
        for (Element param : XMLUtil.elementListByXPath(config, xParameters)) {
            String name         = param.getAttribute("name");
            String display      = param.getAttribute("display");
            String type         = param.getAttribute("type");
            String defaultValue = param.getTextContent();
            
            if (EnumerationUtil.contains(FieldType.class, type)) {
                FieldType fieldType = FieldType.valueOf(type);
                DisplayParameter parm = new DisplayParameter(name, display, fieldType, defaultValue);
                if (parameters == null) {
                    parameters = new HashMap<String, DisplayParameter>();
                }
                parameters.put(name, parm);
            } else {
                OutputUtil.error("[loadConfig] Unknown field type '" + type + "' of parameter '" + name + " for file '"
                                 + configFile + "' using xPath '" + xParameters + "'. Declaration ignored!");
            }
        }
    }
    
    /**
     * This method adds a menu to the user interface.
     */
    protected final void addMenu() {
        if (isStandalone()) {
            JMenu help = new JMenu("Help");
            help.setMnemonic('H');
            
            JMenuItem about = new JMenuItem("About Reachability Agent");
            about.setMnemonic('A');
            about.setName("about");
            help.add(about);
            
            JMenuBar bar = new JMenuBar();
            bar.add(help);
            setJMenuBar(bar);
            
            // Actions
            about.addActionListener(this);
        }
    }
    
    /**
     * Returns a user interface grid.
     * 
     * @param rowsIn        Amount of rows to be displayed
     * @return              Returns a user interface grid.
     */
    private GridLayout getGrid(final int rowsIn) {
        return getGrid(rowsIn, 2);
    }
    
    /**
     * Adds a new row to the users interface, containing the properties type, its value and the 
     * possibility to change it. Supported will be all field types of {@link FieldType}.
     * 
     * @param key           The properties name as {@link String}.
     */
    private void addParameterRow(final String key) {
        if (parameters.containsKey(key)) {
            DisplayParameter param = parameters.get(key);

            addBody(new JLabel(param.getDisplay()));
            
            switch (param.getFieldType()) {
                case FILE_CHOOSER:
                    JTextField field = new JTextField(param.getDefaultValue());
                    field.setEditable(false);
                    field.setName(getConfigValue("PATH_LABEL_PREFIX_FILE_CHOOSER") + param.getName());
                    field.addMouseListener(this);
                    addBody(field);
                    break;
                    
                case INPUT:
                    JTextField inputField = new JTextField(param.getDefaultValue());
                    inputField.setEditable(true);
                    inputField.setName(getConfigValue("FIELD_LABEL_PREFIX_JTEXTFIELD") + param.getName());
                    addBody(inputField);
                    break;
                    
                case INTEGER:
                    JTextField integerField = new JTextField(param.getDefaultValue());
                    integerField.setEditable(true);
                    integerField.setName(getConfigValue("FIELD_LABEL_PREFIX_JTEXTFIELD_INT") + param.getName());
                    integerField.addKeyListener(this);
                    addBody(integerField);
                    break;
                    
                case BOOLEAN:
                    String[] options = {"true", "false"};
                    JComboBox<String> list = new JComboBox<String>(options);
                    list.setCursor(new Cursor(Cursor.HAND_CURSOR));
                    list.setName(getConfigValue("FIELD_LABEL_PREFIX_JCOMBOBOX") + param.getName());
                    if (!param.getDefaultValue().isEmpty()) {
                        list.setSelectedItem(param.getDefaultValue());
                    }
                    addBody(list);
                    break;
                    
                case TEXT:
                default:
                    addBody(new JLabel("Text"));
                
            }
        }
    }
    
    /**
     * Adds a new topic row to the user interface.
     * 
     * @param topic         The topics name as {@link String}.
     */
    private void addTopicRow(final String topic) {
        JLabel top      = new JLabel(topic);
        Font parentFont = getBody().getFont();
        top.setFont(new Font(parentFont.getFamily(), Font.BOLD, parentFont.getSize()));
        
        addBody(top);
        addBody(new JLabel(""));
    }
    
    /**
     * Add a new button to submit the data as row to the user interface.
     */
    private void addSubmitRow() {
        addBody(new JLabel(""));
        
        JButton submit = new JButton("Send");
        submit.setName(getConfigValue("BUTTON_LABEL_PREFIX") + "submit");
        submit.addActionListener(this);
        addBody(submit);
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     * 
     * @param e             An {@link ActionEvent}.
     */
    public final void actionPerformed(final ActionEvent e) {
        // Menu Items
        if (JMenuItem.class.isAssignableFrom(e.getSource().getClass())) {
            String menuItemType = ((JMenuItem) e.getSource()).getName();
            if ("about".equals(menuItemType)) {
                infoDialog("Version: " + MainConfig.get("REACHABILITY_UI_VERSION")
                         + "\nAuthor: " + ProjectInfo.REACHABILITY_AGENT_AUTHOR);
            }
        }
        
        // Buttons
        if (JButton.class.isAssignableFrom(e.getSource().getClass())) {
            JButton button = (JButton) e.getSource();
            
            // Start?
            if ((getConfigValue("BUTTON_LABEL_PREFIX") + "submit").equals(button.getName())) {
                Map<String, String> params = new HashMap<String, String>();
                
                for (int i = 0; i < getBody().getComponentCount(); i++) {
                    // Selectbox found
                    if (JComboBox.class.isAssignableFrom(getBody().getComponent(i).getClass())) {
                        @SuppressWarnings("unchecked")
                        JComboBox<String> list  = (JComboBox<String>) getBody().getComponent(i);
                        String key              = getConfigValue("FIELD_LABEL_PREFIX_JCOMBOBOX");
                        String parameter        = list.getName().substring(key.length());
                        String value            = (String) list.getSelectedItem();
                        
                        params.put(parameter, value);
                    }
                    
                    // TextField found
                    if (JTextField.class.isAssignableFrom(getBody().getComponent(i).getClass())) {
                        JTextField field = (JTextField) getBody().getComponent(i);
                        
                        String[] keys = {getConfigValue("PATH_LABEL_PREFIX_FILE_CHOOSER"),
                                         getConfigValue("FIELD_LABEL_PREFIX_JTEXTFIELD_INT"),
                                         getConfigValue("FIELD_LABEL_PREFIX_JTEXTFIELD")};
                        for (String key : keys) {
                            if (field.getName().startsWith(key)) {
                                String parameter = field.getName().substring(key.length());
                                String value     = field.getText();
                                                                
                                if (!value.isEmpty()) {
                                    params.put(parameter, value);
                                }
                            }
                        }
                    }
                }

                // Send new configuration order
                ConfigOrder x = ConfigOrder.valueOf(params.get("order"));
                if (x != null) {
                    JsonConfigOrder order = new JsonConfigOrder();
                    
                    order.Id        = params.get("id");
                    order.order     = x;
                    order.Version   = ProjectInfo.REACHABILITY_AGENT_VERSION;
                    
                    ConnectionUtil.setMessageOnTopic(globalControlTopic, order);
                }
            }
        }
    }
    
    /**
     * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
     * 
     * @param e             A {@link MouseEvent}.
     */
    public final void mousePressed(final MouseEvent e) {
        // JTextFields
        if (JTextField.class.isAssignableFrom(e.getSource().getClass())) {
            JTextField field = (JTextField) e.getSource();
            
            // File Chooser
            if (field.getName().startsWith(getConfigValue("PATH_LABEL_PREFIX_FILE_CHOOSER"))) {
                JFileChooser x = new JFileChooser();
                if (!field.getText().isEmpty()) {
                    x.setSelectedFile(new File(field.getText()));
                }
                                    
                int result = x.showOpenDialog(ControllingToolUI.this);
                if (result == JFileChooser.APPROVE_OPTION) {
                    String file = x.getSelectedFile().getAbsolutePath();
                    field.setText(file);
                }
            }
        }
    }

    /**
     * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
     * 
     * @param arg0          A {@link KeyEvent}.
     */
    public final void keyReleased(final KeyEvent arg0) {
        // JTextFields
        if (JTextField.class.isAssignableFrom(arg0.getSource().getClass())) {
            JTextField field = (JTextField) arg0.getSource();
            
            // Integer only
            if (field.getName().startsWith(getConfigValue("FIELD_LABEL_PREFIX_JTEXTFIELD_INT"))) {
                // Check content
                field.setText(field.getText().replaceAll("[^0-9]", ""));
            }
        }
    }
    
    /**
     * @see de.hawhamburg.reachability.StopableObject#shutDown()
     */
    public final void shutDown() {
        if (this.globalControlTopic != null) {
            this.globalControlTopic.disconnect();
        }
    }
    
    /**
     * Builds the user interface. The rendering will not be influenced by the <i>refresh</i> parameter.
     * 
     * @param refresh   Indicates if the call was meant as initial build or refresh.
     */
    public final void refresh(final boolean refresh) {
        int rows = 1 + parameters.size() + 2;
        int newHeight = getConfigValueAsInteger("DEFAULT_WINDOW_MENU_HEIGHT")
                        + (getConfigValueAsInteger("DEFAULT_ROW_HEIGHT") * rows);
        this.setWindowHeight(newHeight);
        fixBounds(refresh);
        
        // Define layout
        getBody().setLayout(getGrid(rows));
        getContentPane().add(getBody());
        getBody().removeAll();
        
        // Set configuration rows
        addTopicRow("Parameter:");
        for (String param : parameters.keySet()) {
            addParameterRow(param);
        }
        addTopicRow("");
        addSubmitRow();
            
        // Reload content
        fixBounds(refresh);
        getBody().revalidate();
    }
    
    /**
     * Creates and starts a new {@link ControllingToolUI} in systems look and feel.
     * 
     * @param args      The systems supplied start agruments as array of {@link String}.
     */
    public static void main(final String[] args) {
        // Remove all existing loggers
        org.apache.log4j.Logger.getRootLogger().removeAllAppenders();
        
        // Try to set to systems look and feel
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (ClassNotFoundException e) {
            OutputUtil.error("[ClassNotFoundException] " + e.getMessage());
        } catch (InstantiationException e) {
            OutputUtil.error("[InstantiationException] " + e.getMessage());
        } catch (IllegalAccessException e) {
            OutputUtil.error("[IllegalAccessException] " + e.getMessage());
        } catch (UnsupportedLookAndFeelException e) {
            OutputUtil.error("[UnsupportedLookAndFeelException] " + e.getMessage());
        }
        
        // Initialize a new controlling tool user interface
        new ControllingToolUI();
    }
}
