package de.hawhamburg.reachability;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.hawhamburg.lifecycle.agent.LifeCycleAgent;
import de.hawhamburg.lifecycle.agent.LifeCycleApp;
import de.hawhamburg.lifecycle.data.JsonInfoMessage;
import de.hawhamburg.lifecycle.data.ProjectStatus;
import de.hawhamburg.reachability.util.OutputUtil;

/**
 * This is the main class for starting up the reachability agent. It will open a new ReachabilityWorker an start it on 
 * request. It also implements the living place (LP) life cycle agent, so it listens on the messages "start", "sleep", 
 * "info" and "stop" via the global control topic.
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR}
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public class ReachabilityAgent implements LifeCycleAgent {
    
    /**
     * Time to wait before check again if the worker has already been stopped, while {@link #stop()} is ending the 
     * worker thread, before sending success.  
     */
    private static final long SHUTDOWN_CHECKTIME    = 100;
    
    /**
     * The {@link ReachabilityWorker}, a thread determining the current state of reachability of an inhabitant.
     */
    private ReachabilityWorker worker               = null;
    
    /**
     * The current project status of the reachability agent as {@link ProjectStatus} object.
     */
    private ProjectStatus myStatus;

    /**
     * Creates a new {@link ReachabilityAgent} which starts a {@link ReachabilityWorker} to determine the inhabitants
     * current state of reachability. 
     */
    public ReachabilityAgent() {
        // Logout start to console
        OutputUtil.info("*******************************");
        OutputUtil.info("** New life cycle agent '" + this.getClass().getSimpleName() + "' started");
        OutputUtil.info("*******************************");
        OutputUtil.info("** - ID:            '" + this.getID() + "'");
        OutputUtil.info("** - Current state: '" + ProjectStatus.SLEEPING + "'");
        OutputUtil.info("*******************************");
        
        // Create and start a new worker thread
        this.worker = new ReachabilityWorker();
        this.worker.start();
        
        // Wrap this thread by a life cycle app
        LifeCycleApp lifecycle = new LifeCycleApp();
        lifecycle.setLifeCycleAgent(this);
        new Thread(lifecycle).start();
        
        // Set project status sleeping
        this.myStatus = ProjectStatus.SLEEPING;
    }
    
    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#sleep()
     */
    public final void sleep() {
        OutputUtil.info("Getting Sleeping Order");
        this.myStatus = ProjectStatus.SLEEPING;
        this.worker.activ(false);
    }

    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#getInfoMessage()
     * 
     * @return      A {@link JsonInfoMessage} object describing the current systems state.
     */
    public final JsonInfoMessage getInfoMessage() {
        JsonInfoMessage msg = new JsonInfoMessage();

        msg.author  = ProjectInfo.REACHABILITY_AGENT_AUTHOR;
        msg.Id      = this.getID();
        msg.status  = this.myStatus.toString();
        msg.Version = ProjectInfo.REACHABILITY_AGENT_VERSION;
        try {
            msg.ip = InetAddress.getLocalHost().toString();
        } catch (UnknownHostException e) {
            Logger.getLogger(ReachabilityAgent.class.getName()).log(Level.SEVERE, null, e);
            msg.ip = "unknown";
        }
        msg.misc    = ProjectInfo.REACHABILITY_AGENT_DESCRIPTION;
        
        OutputUtil.info("Sending INFO Message");
        
        return msg;
    }

    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#start()
     */
    public final void start() {
        OutputUtil.info("Getting Starting Order!");
        this.myStatus = ProjectStatus.RUNNING;
        
        this.worker.activ(true);
    }

    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#stop()
     */
    public final void stop() {
        OutputUtil.info("Getting Stopping Order!");
        this.myStatus = ProjectStatus.STOPPED;
        
        if (this.worker != null) {
            this.worker.shutDown();
            while (!this.worker.ended()) {
                try {
                    Thread.sleep(SHUTDOWN_CHECKTIME);
                } catch (InterruptedException e) {
                    System.err.println(this.getClass().getName() + ": Unable to sleep this thread. " + e.getMessage());
                }
            }
            this.worker = null;
        }

        OutputUtil.info("Worker has stopped!");
    }

    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#getID()
     * 
     * @return      The ID of this service as {@link String} defined by
     *              {@link ProjectInfo#REACHABILITY_AGENT_NAME}.
     */
    public final String getID() {
        return ProjectInfo.REACHABILITY_AGENT_NAME;
    }
    
    /**
     * This method starts and runs a new {@link ReachabilityAgent}.
     * 
     * @param args  The command line arguments as array of {@link String}.
     */
    public static void main(final String[] args) {
        // Remove all existing loggers
        org.apache.log4j.Logger.getRootLogger().removeAllAppenders();
        
        new ReachabilityAgent();
    }
}
